//////////////////////////////////////////////////////////////////////////////////////////////
//
// 	Classe : masterm v1.0
//	(c)S.ANTOINE - 2022
//
//	Petit MASTER-MIND à améliorer
//
//	Constructeur : Fournir l'identifiant CSS d'une DIV cible
//	Fichiers associés : masterm.js et masterm.css
//////////////////////////////////////////////////////////////////////////////////////////////


class masterm {
	constructor (cible) {
		this.target = document.querySelector(cible); // L'objet DOM HTML qui contiendra le jeu
		this.couleurs = ['yellow', 'blue', 'red', 'green', 'white', 'black'];
		this.mystery = [];	// Pour stocker les couleurs à trouver
		this.lastLine = 0;	// Pour conserver l'accès à la dernière ligne
		this.nbLines = 0;	// Pour identifier la ligne
		this.nbJetons = 5;	// Le nombre de jetons à trouver
		
		this.addLine(); 		// On commence par ajouter une ligne
		this.getNewMystery();	// On calcule
	}
	
	/// ///////////////////////////////////////////////////////////////////////////////
	/// addLine : Ajouter une ligne de jetons + jetons de controle
	/// ///////////////////////////////////////////////////////////////////////////////
	addLine() {
		this.lastLine = document.createElement('div'); // Création d'un objet DOM de type <div>
		this.lastLine.setAttribute('class', 'ligne');  // Décoration CSS
		
		this.nbLines++; // Compteur de ligne
		this.lastLine.id = this.nbLines; // Identification de la ligne
		
		// Contenu de la balise : les jetons et le controle
		for (let i=0; i<this.nbJetons; i++) {
			this.lastLine.innerHTML += "<div class='rond'></div>";
		}
		for (let i=0; i<this.nbJetons; i++) {
			this.lastLine.innerHTML += "<div class='controle'></div>";
		}
		
		// Ajout des événements pour chaque <div class='rond'>
		this.lastLine.querySelectorAll('.rond').forEach( (rond) => {
			rond.addEventListener('click', () => {
				/// Action à réaliser à chaque click :
				// Cas d'une ancienne ligne : on ne doit pas la modifier : on quitte
				if ( this.nbLines > rond.parentNode.id) return; 
				
				// Cas normal : On change la couleur du jeton
				let c = rond.style.backgroundColor; // Récupère la couleur actuelle
				let index = this.couleurs.findIndex((color) => color == c); // Récupère sa position dans le tableau des couleurs
				index ++; // index couleur suivante du tableau des couleurs
				if (index >= this.couleurs.length) index = 0; // Pour ne pas dépasser le tableau des couleurs
				rond.style.backgroundColor = this.couleurs[index]; // On applique la nouvelle couleur
			});
		});
		
		this.target.appendChild(this.lastLine); // On ajoute la ligne au document HTML
	}
	
	/// ///////////////////////////////////////////////////////////////////////////////
	/// Remplir le tableau des couleurs à trouver
	/// ///////////////////////////////////////////////////////////////////////////////
	getNewMystery() {
		for (let i=0; i<this.nbJetons; i++)
		{
			let nombreCouleurs = this.couleurs.length - 1; 
			let i = Math.floor(Math.random() * nombreCouleurs); // Nombre aléatoire entre 0 et nombre de couleurs
			this.mystery.push(this.couleurs[i]); // On ajoute la couleur mystère au tableau
		}
	}
	
	/// ///////////////////////////////////////////////////////////////////////////////
	/// Teste la solution proposée et modifie les couleurs des jetons de contrôle.
	/// ///////////////////////////////////////////////////////////////////////////////
	getResultat() {
		let L = this.lastLine.querySelectorAll('.rond'); // Tableau des jetons de la dernière ligne
		let C = this.lastLine.querySelectorAll('.controle'); // Tableau des jetons de controle de la dernière ligne
		let controle = 0; // Utilisé pour savoir si on a gagné
		
		// Pour chaque jeton, on controle s'il est à la bonne place ou pas
		L.forEach( (col, pos) => {
			let res = this.testColor(col.style.backgroundColor, pos);
			// Modification de la couleur des jetons de controle
			if ( res == 0 ) C[pos].style.backgroundColor = 'white'; // Bien Placé
			if ( res == 1 ) C[pos].style.backgroundColor = 'black'; // Mal placé
			
			controle += res; // Astuce de math : si tout est bon, res reste à 0
		});
		
		if (controle == 0) { // On vérifie si on a tout trouvé
			alert("Gagné !!");
		}
		else { // Sinon on ajoute une nouvelle ligne
			this.addLine();
		}
	}
	
	/// ///////////////////////////////////////////////////////////////////////////////
	/// Méthode qui teste 1 couleur et 1 position
	/// Valeur de retour : un nombre entre 0 et 2
	/// ///////////////////////////////////////////////////////////////////////////////
	testColor(color, position) {
		if (this.mystery[position] == color ) // Test : La couleur est-elle au bon endroit dans la tableau mystère ?
			return 0;// Bien placé, on quitte
		if (this.mystery.find( (c) => c == color) === undefined) // Test: la couleur proposée est-elle dans le tableau mystère ?
			return 2;// n'existe pas, on quitte
		else
			return 1;// sinon : pas bien placé, on quitte
	}
}